@@ -184,16 +184,16 @@ parse_nfo(const char *path, metadata_t *m)
 	val = GetValueFromNameValueList(&xml, "title");
 	if( val )
 	{
-		char *esc_tag = unescape_tag(val, 1);
+		char *esc_tag, *title;
 		val2 = GetValueFromNameValueList(&xml, "episodetitle");
-		if( val2 ) {
-			char *esc_tag2 = unescape_tag(val2, 1);
-			xasprintf(&m->title, "%s - %s", esc_tag, esc_tag2);
-			free(esc_tag2);
-		} else {
-			m->title = escape_tag(esc_tag, 1);
-		}
+		if( val2 )
+			xasprintf(&title, "%s - %s", val, val2);
+		else
+			title = strdup(val);
+		esc_tag = unescape_tag(title, 1);
+		m->title = escape_tag(esc_tag, 1);
 		free(esc_tag);
+		free(title);
 	}
 
 	val = GetValueFromNameValueList(&xml, "plot");
@@ -381,7 +381,7 @@ GetAudioMetadata(const char *path, char *name)
 	{
 		m.title = name;
 	}
-	for( i=ROLE_START; i<N_ROLE; i++ )
+	for( i = ROLE_START; i < N_ROLE; i++ )
 	{
 		if( song.contributor[i] && *song.contributor[i] )
 		{
@@ -400,12 +400,17 @@ GetAudioMetadata(const char *path, char *name)
 			break;
 		}
 	}
-	/* If there is a band associated with the album, use it for virtual containers. */
-	if( (i != ROLE_BAND) && (i != ROLE_ALBUMARTIST) )
+	/* If there is a album artist or band associated with the album,
+	   use it for virtual containers. */
+	if( i < ROLE_ALBUMARTIST )
 	{
-	        if( song.contributor[ROLE_BAND] && *song.contributor[ROLE_BAND] )
+		for( i = ROLE_ALBUMARTIST; i <= ROLE_BAND; i++ )
+		{
+			if( song.contributor[i] && *song.contributor[i] )
+				break;
+		}
+	        if( i <= ROLE_BAND )
 		{
-			i = ROLE_BAND;
 			m.artist = trim(song.contributor[i]);
 			if( strlen(m.artist) > 48 )
 			{
@@ -689,16 +694,16 @@ GetVideoMetadata(const char *path, char *name)
 	//dump_format(ctx, 0, NULL, 0);
 	for( i=0; i<ctx->nb_streams; i++)
 	{
-		if( audio_stream == -1 &&
-		    ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO )
+		if( ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO &&
+		    audio_stream == -1 )
 		{
 			audio_stream = i;
 			ac = ctx->streams[audio_stream]->codec;
 			continue;
 		}
-		else if( video_stream == -1 &&
+		else if( ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO &&
 		         !lav_is_thumbnail_stream(ctx->streams[i], &m.thumb_data, &m.thumb_size) &&
-			 ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO )
+		         video_stream == -1 )
 		{
 			video_stream = i;
 			vc = ctx->streams[video_stream]->codec;
