@@ -372,7 +372,7 @@ rescan:
 #if USE_FORK
 		scanning = 1;
 		sqlite3_close(db);
-		*scanner_pid = process_fork();
+		*scanner_pid = fork();
 		open_db(&db);
 		if (*scanner_pid == 0) /* child (scanner) process */
 		{
@@ -380,6 +380,7 @@ rescan:
 			sqlite3_close(db);
 			log_close();
 			freeoptions();
+			free(children);
 			exit(EXIT_SUCCESS);
 		}
 		else if (*scanner_pid < 0)
@@ -464,6 +465,16 @@ static int strtobool(const char *str)
 		(atoi(str) == 1));
 }
 
+static void init_nls(void)
+{
+#ifdef ENABLE_NLS
+	setlocale(LC_MESSAGES, "");
+	setlocale(LC_CTYPE, "en_US.utf8");
+	DPRINTF(E_DEBUG, L_GENERAL, "Using locale dir %s\n", bindtextdomain("minidlna", getenv("TEXTDOMAINDIR")));
+	textdomain("minidlna");
+#endif
+}
+
 /* init phase :
  * 1) read configuration file
  * 2) read command line arguments
@@ -544,6 +555,8 @@ init(int argc, char **argv)
 						MAX_LAN_ADDR, word);
 					break;
 				}
+				while (isspace(*word))
+					word++;
 				runtime_vars.ifaces[ifaces++] = word;
 			}
 			break;
@@ -710,7 +723,8 @@ init(int argc, char **argv)
 				/* Symbolic username given, not UID. */
 				struct passwd *entry = getpwnam(ary_options[i].value);
 				if (!entry)
-					DPRINTF(E_FATAL, L_GENERAL, "Bad user '%s'.\n", argv[i]);
+					DPRINTF(E_FATAL, L_GENERAL, "Bad user '%s'.\n",
+						ary_options[i].value);
 				uid = entry->pw_uid;
 			}
 			break;
@@ -969,6 +983,13 @@ init(int argc, char **argv)
 		DPRINTF(E_FATAL, L_GENERAL, "Failed to switch to uid '%d'. [%s] EXITING.\n",
 			uid, strerror(errno));
 
+	children = calloc(runtime_vars.max_connections, sizeof(struct child));
+	if (!children)
+	{
+		DPRINTF(E_ERROR, L_GENERAL, "Allocation failed\n");
+		return 1;
+	}
+
 	return 0;
 }
 
@@ -1000,12 +1021,7 @@ main(int argc, char **argv)
 
 	for (i = 0; i < L_MAX; i++)
 		log_level[i] = E_WARN;
-#ifdef ENABLE_NLS
-	setlocale(LC_MESSAGES, "");
-	setlocale(LC_CTYPE, "en_US.utf8");
-	DPRINTF(E_DEBUG, L_GENERAL, "Using locale dir %s\n", bindtextdomain("minidlna", getenv("TEXTDOMAINDIR")));
-	textdomain("minidlna");
-#endif
+	init_nls();
 
 	ret = init(argc, argv);
 	if (ret != 0)
@@ -1043,6 +1059,7 @@ main(int argc, char **argv)
 	if (sssdp < 0)
 	{
 		DPRINTF(E_INFO, L_GENERAL, "Failed to open socket for receiving SSDP. Trying to use MiniSSDPd\n");
+		reload_ifaces(0);	/* populate lan_addr[0].str */
 		if (SubmitServicesToMiniSSDPD(lan_addr[0].str, runtime_vars.port) < 0)
 			DPRINTF(E_FATAL, L_GENERAL, "Failed to connect to MiniSSDPd. EXITING");
 	}
@@ -1182,11 +1199,6 @@ main(int argc, char **argv)
 				i++;
 			}
 		}
-#ifdef DEBUG
-		/* for debug */
-		if (i > 1)
-			DPRINTF(E_DEBUG, L_GENERAL, "%d active incoming HTTP connections\n", i);
-#endif
 		FD_ZERO(&writeset);
 		upnpevents_selectfds(&readset, &writeset, &max_fd);
 
@@ -1285,7 +1297,11 @@ main(int argc, char **argv)
 shutdown:
 	/* kill the scanner */
 	if (scanning && scanner_pid)
-		kill(scanner_pid, 9);
+		kill(scanner_pid, SIGKILL);
+
+	/* kill other child processes */
+	process_reap_children();
+	free(children);
 
 	/* close out open sockets */
 	while (upnphttphead.lh_first != NULL)
@@ -1298,10 +1314,12 @@ shutdown:
 		close(sssdp);
 	if (shttpl >= 0)
 		close(shttpl);
-	#ifdef TIVO_SUPPORT
+#ifdef TIVO_SUPPORT
 	if (sbeacon >= 0)
 		close(sbeacon);
-	#endif
+#endif
+	if (smonitor >= 0)
+		close(smonitor);
 	
 	for (i = 0; i < n_lan_addr; i++)
 	{
