@@ -49,20 +49,18 @@
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <sys/param.h>
+#include <limits.h>
 
 #include "config.h"
 #include "upnpglobalvars.h"
 
-/* LAN address */
-/*const char * listen_addr = 0;*/
-
 /* startup time */
 time_t startup_time = 0;
 
 struct runtime_vars_s runtime_vars;
 uint32_t runtime_flags = INOTIFY_MASK;
 
-const char * pidfilename = "/var/run/minidlna/minidlna.pid";
+const char *pidfilename = "/var/run/minidlna/minidlna.pid";
 
 char uuidvalue[] = "uuid:00000000-0000-0000-0000-000000000000";
 char modelname[MODELNAME_MAX_LEN] = ROOTDEV_MODELNAME;
@@ -78,12 +76,13 @@ char presentationurl[PRESENTATIONURL_MAX_LEN];
 
 int n_lan_addr = 0;
 struct lan_addr_s lan_addr[MAX_LAN_ADDR];
+int sssdp = -1;
 
 /* Path of the Unix socket used to communicate with MiniSSDPd */
 const char * minissdpdsocketpath = "/var/run/minissdpd.sock";
 
 /* UPnP-A/V [DLNA] */
-sqlite3 * db;
+sqlite3 *db;
 char friendly_name[FRIENDLYNAME_MAX_LEN];
 char db_path[PATH_MAX] = {'\0'};
 char log_path[PATH_MAX] = {'\0'};
@@ -92,3 +91,4 @@ struct album_art_name_s * album_art_names = NULL;
 short int scanning = 0;
 volatile short int quitting = 0;
 volatile uint32_t updateID = 0;
+const char *force_sort_criteria = NULL;
