@@ -80,13 +80,13 @@
 #include "tivo_commands.h"
 #include "clients.h"
 #include "process.h"
-
 #include "sendfile.h"
 
-//#define MAX_BUFFER_SIZE 4194304 // 4MB -- Too much?
-#define MAX_BUFFER_SIZE 2147483647 // 2GB -- Too much?
+#define MAX_BUFFER_SIZE 2147483647
 #define MIN_BUFFER_SIZE 65536
 
+#define INIT_STR(s, d) { s.data = d; s.size = sizeof(d); s.off = 0; }
+
 #include "icons.c"
 
 enum event_type {
@@ -95,6 +95,13 @@ enum event_type {
 	E_RENEW
 };
 
+static void SendResp_icon(struct upnphttp *, char * url);
+static void SendResp_albumArt(struct upnphttp *, char * url);
+static void SendResp_caption(struct upnphttp *, char * url);
+static void SendResp_resizedimg(struct upnphttp *, char * url);
+static void SendResp_thumbnail(struct upnphttp *, char * url);
+static void SendResp_dlnafile(struct upnphttp *, char * url);
+
 struct upnphttp * 
 New_upnphttp(int s)
 {
@@ -137,6 +144,7 @@ Delete_upnphttp(struct upnphttp * h)
 static void
 ParseHttpHeaders(struct upnphttp * h)
 {
+	int client = 0;
 	char * line;
 	char * colon;
 	char * p;
@@ -154,6 +162,10 @@ ParseHttpHeaders(struct upnphttp * h)
 				while(*p && (*p < '0' || *p > '9'))
 					p++;
 				h->req_contentlen = atoi(p);
+				if(h->req_contentlen < 0) {
+					DPRINTF(E_WARN, L_HTTP, "Invalid Content-Length %d", h->req_contentlen);
+					h->req_contentlen = 0;
+				}
 			}
 			else if(strncasecmp(line, "SOAPAction", 10)==0)
 			{
@@ -274,7 +286,7 @@ ParseHttpHeaders(struct upnphttp * h)
 			{
 				int i;
 				/* Skip client detection if we already detected it. */
-				if( h->req_client )
+				if( client )
 					goto next_header;
 				p = colon + 1;
 				while(isspace(*p))
@@ -285,7 +297,7 @@ ParseHttpHeaders(struct upnphttp * h)
 						continue;
 					if (strstrc(p, client_types[i].match, '\r') != NULL)
 					{
-						h->req_client = i;
+						client = i;
 						break;
 					}
 				}
@@ -294,7 +306,7 @@ ParseHttpHeaders(struct upnphttp * h)
 			{
 				int i;
 				/* Skip client detection if we already detected it. */
-				if( h->req_client && client_types[h->req_client].type < EStandardDLNA150 )
+				if( client && client_types[client].type < EStandardDLNA150 )
 					goto next_header;
 				p = colon + 1;
 				while(isspace(*p))
@@ -305,7 +317,7 @@ ParseHttpHeaders(struct upnphttp * h)
 						continue;
 					if (strstrc(p, client_types[i].match, '\r') != NULL)
 					{
-						h->req_client = i;
+						client = i;
 						break;
 					}
 				}
@@ -386,7 +398,7 @@ ParseHttpHeaders(struct upnphttp * h)
 						continue;
 					if (strstrc(p, client_types[i].match, '\r') != NULL)
 					{
-						h->req_client = i;
+						client = i;
 						break;
 					}
 				}
@@ -400,9 +412,7 @@ ParseHttpHeaders(struct upnphttp * h)
 next_header:
 		line = strstr(line, "\r\n");
 		if (!line)
-		{
 			return;
-		}
 		line += 2;
 	}
 	if( h->reqflags & FLAG_CHUNKED )
@@ -432,29 +442,24 @@ next_header:
 	/* If the client type wasn't found, search the cache.
 	 * This is done because a lot of clients like to send a
 	 * different User-Agent with different types of requests. */
-	n = SearchClientCache(h->clientaddr, 0);
+	h->req_client = SearchClientCache(h->clientaddr, 0);
 	/* Add this client to the cache if it's not there already. */
-	if( n < 0 )
+	if (!h->req_client)
 	{
-		AddClientCache(h->clientaddr, h->req_client);
+		h->req_client = AddClientCache(h->clientaddr, client);
 	}
-	else if (h->req_client)
+	else if (client)
 	{
-		enum client_types type = client_types[h->req_client].type;
-		enum client_types ctype = client_types[clients[n].type].type;
+		enum client_types type = client_types[client].type;
+		enum client_types ctype = h->req_client->type->type;
 		/* If we know the client and our new detection is generic, use our cached info */
 		/* If we detected a Samsung Series B earlier, don't overwrite it with Series A info */
 		if ((ctype && ctype < EStandardDLNA150 && type >= EStandardDLNA150) ||
 		    (ctype == ESamsungSeriesB && type == ESamsungSeriesA))
-		{
-			h->req_client = clients[n].type;
 			return;
-		}
-		clients[n].type = h->req_client;
-		clients[n].age = time(NULL);
+		h->req_client->type = &client_types[client];
+		h->req_client->age = time(NULL);
 	}
-	else
-		h->req_client = clients[n].type;
 }
 
 /* very minimalistic 400 error message */
@@ -590,9 +595,7 @@ SendResp_presentation(struct upnphttp * h)
 	char body[4096];
 	int a, v, p, i;
 
-	str.data = body;
-	str.size = sizeof(body);
-	str.off = 0;
+	INIT_STR(str, body);
 
 	h->respflags = FLAG_HTML;
 
@@ -619,17 +622,20 @@ SendResp_presentation(struct upnphttp * h)
 	strcatf(&str,
 		"<h3>Connected clients</h3>"
 		"<table border=1 cellpadding=10>"
-		"<tr><td>ID</td><td>Type</td><td>IP Address</td><td>HW Address</td></tr>");
+		"<tr><td>ID</td><td>Type</td><td>IP Address</td><td>HW Address</td><td>Connections</td></tr>");
 	for (i = 0; i < CLIENT_CACHE_SLOTS; i++)
 	{
 		if (!clients[i].addr.s_addr)
 			continue;
-		strcatf(&str, "<tr><td>%d</td><td>%s</td><td>%s</td><td>%02X:%02X:%02X:%02X:%02X:%02X</td></tr>",
-				i, client_types[clients[i].type].name, inet_ntoa(clients[i].addr),
+		strcatf(&str, "<tr><td>%d</td><td>%s</td><td>%s</td><td>%02X:%02X:%02X:%02X:%02X:%02X</td><td>%d</td></tr>",
+				i, clients[i].type->name, inet_ntoa(clients[i].addr),
 				clients[i].mac[0], clients[i].mac[1], clients[i].mac[2],
-				clients[i].mac[3], clients[i].mac[4], clients[i].mac[5]);
+				clients[i].mac[3], clients[i].mac[4], clients[i].mac[5], clients[i].connections);
 	}
-	strcatf(&str, "</table></BODY></HTML>\r\n");
+	strcatf(&str, "</table>");
+
+	strcatf(&str, "<br>%d connection%s currently open<br>", number_of_children, (number_of_children == 1 ? "" : "s"));
+	strcatf(&str, "</BODY></HTML>\r\n");
 
 	BuildResp_upnphttp(h, str.data, str.off);
 	SendResp_upnphttp(h);
@@ -821,8 +827,6 @@ ProcessHttpQuery_upnphttp(struct upnphttp * h)
 	for(i = 0; i<15 && *p && *p != '\r'; i++)
 		HttpVer[i] = *(p++);
 	HttpVer[i] = '\0';
-	/*DPRINTF(E_INFO, L_HTTP, "HTTP REQUEST : %s %s (%s)\n",
-	       HttpCommand, HttpUrl, HttpVer);*/
 
 	/* set the interface here initially, in case there is no Host header */
 	for(i = 0; i<n_lan_addr; i++)
@@ -907,7 +911,7 @@ ProcessHttpQuery_upnphttp(struct upnphttp * h)
 		if(strcmp(ROOTDESC_PATH, HttpUrl) == 0)
 		{
 			/* If it's a Xbox360, we might need a special friendly_name to be recognized */
-			if( client_types[h->req_client].type == EXbox )
+			if( h->req_client && h->req_client->type->type == EXbox )
 			{
 				char model_sav[2];
 				i = 0;
@@ -923,7 +927,7 @@ ProcessHttpQuery_upnphttp(struct upnphttp * h)
 					friendly_name[i] = '\0';
 				memcpy(modelnumber, model_sav, 2);
 			}
-			else if( client_types[h->req_client].flags & FLAG_SAMSUNG_DCM10 )
+			else if( h->req_client && h->req_client->type->flags & FLAG_SAMSUNG_DCM10 )
 			{
 				sendXMLdesc(h, genRootDescSamsung);
 			}
@@ -1176,9 +1180,6 @@ BuildHeader_upnphttp(struct upnphttp * h, int respcode,
 	strftime(date, 30,"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&curtime));
 	strcatf(&res, "Date: %s\r\n", date);
 	strcatf(&res, "EXT:\r\n");
-#if 0 // DLNA
-	strcatf(&res, "contentFeatures.dlna.org: \r\n");
-#endif
 	strcatf(&res, "\r\n");
 	h->res_buflen = res.off;
 	if(h->res_buf_alloclen < (h->res_buflen + bodylen))
@@ -1226,7 +1227,7 @@ SendResp_upnphttp(struct upnphttp * h)
 	}
 }
 
-int
+static int
 send_data(struct upnphttp * h, char * header, size_t size, int flags)
 {
 	int n;
@@ -1249,7 +1250,7 @@ send_data(struct upnphttp * h, char * header, size_t size, int flags)
 	return 1;
 }
 
-void
+static void
 send_file(struct upnphttp * h, int sendfd, off_t offset, off_t end_offset)
 {
 	off_t send_size;
@@ -1259,7 +1260,7 @@ send_file(struct upnphttp * h, int sendfd, off_t offset, off_t end_offset)
 	int try_sendfile = 1;
 #endif
 
-	while( offset < end_offset )
+	while( offset <= end_offset )
 	{
 #if HAVE_SENDFILE
 		if( try_sendfile )
@@ -1308,15 +1309,33 @@ send_file(struct upnphttp * h, int sendfd, off_t offset, off_t end_offset)
 	free(buf);
 }
 
-void
+static void
+start_dlna_header(struct string_s *str, int respcode, const char *tmode, const char *mime)
+{
+	char date[30];
+	time_t now;
+
+	now = time(NULL);
+	strftime(date, sizeof(date),"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&now));
+	strcatf(str, "HTTP/1.1 %d OK\r\n"
+	             "Connection: close\r\n"
+	             "Date: %s\r\n"
+	             "Server: " MINIDLNA_SERVER_STRING "\r\n"
+	             "EXT:\r\n"
+	             "realTimeInfo.dlna.org: DLNA.ORG_TLAG=*\r\n"
+	             "transferMode.dlna.org: %s\r\n"
+	             "Content-Type: %s\r\n",
+	             respcode, date, tmode, mime);
+}
+
+static void
 SendResp_icon(struct upnphttp * h, char * icon)
 {
 	char header[512];
 	char mime[12] = "image/";
-	char date[30];
 	char *data;
-	int size, ret;
-	time_t curtime = time(NULL);
+	int size;
+	struct string_s str;
 
 	if( strcmp(icon, "sm.png") == 0 )
 	{
@@ -1353,16 +1372,12 @@ SendResp_icon(struct upnphttp * h, char * icon)
 		return;
 	}
 
-	strftime(date, 30,"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&curtime));
-	ret = snprintf(header, sizeof(header), "HTTP/1.1 200 OK\r\n"
-	                                       "Content-Type: %s\r\n"
-	                                       "Content-Length: %d\r\n"
-	                                       "Connection: close\r\n"
-	                                       "Date: %s\r\n"
-	                                       "Server: " MINIDLNA_SERVER_STRING "\r\n\r\n",
-	                                       mime, size, date);
+	INIT_STR(str, header);
 
-	if( send_data(h, header, ret, MSG_MORE) == 0 )
+	start_dlna_header(&str, 200, "Interactive", mime);
+	strcatf(&str, "Content-Length: %d\r\n\r\n", size);
+
+	if( send_data(h, str.data, str.off, MSG_MORE) == 0 )
 	{
 		if( h->req_command != EHead )
 			send_data(h, data, size, 0);
@@ -1370,16 +1385,15 @@ SendResp_icon(struct upnphttp * h, char * icon)
 	CloseSocket_upnphttp(h);
 }
 
-void
+static void
 SendResp_albumArt(struct upnphttp * h, char * object)
 {
 	char header[512];
 	char *path;
-	char date[30];
-	time_t curtime = time(NULL);
 	off_t size;
 	long long id;
-	int fd, ret;
+	int fd;
+	struct string_s str;
 
 	if( h->reqflags & (FLAG_XFERSTREAMING|FLAG_RANGE) )
 	{
@@ -1410,20 +1424,14 @@ SendResp_albumArt(struct upnphttp * h, char * object)
 	size = lseek(fd, 0, SEEK_END);
 	lseek(fd, 0, SEEK_SET);
 
-	strftime(date, 30,"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&curtime));
-	ret = snprintf(header, sizeof(header), "HTTP/1.1 200 OK\r\n"
-	                                       "Content-Type: image/jpeg\r\n"
-	                                       "Content-Length: %jd\r\n"
-	                                       "Connection: close\r\n"
-	                                       "Date: %s\r\n"
-	                                       "EXT:\r\n"
-	                                       "realTimeInfo.dlna.org: DLNA.ORG_TLAG=*\r\n"
-	                                       "contentFeatures.dlna.org: DLNA.ORG_PN=JPEG_TN\r\n"
-	                                       "Server: " MINIDLNA_SERVER_STRING "\r\n"
-	                                       "transferMode.dlna.org: Interactive\r\n\r\n",
-	                                       (intmax_t)size, date);
+	INIT_STR(str, header);
 
-	if( send_data(h, header, ret, MSG_MORE) == 0 )
+	start_dlna_header(&str, 200, "Interactive", "image/jpeg");
+	strcatf(&str, "Content-Length: %jd\r\n"
+	              "contentFeatures.dlna.org: DLNA.ORG_PN=JPEG_TN\r\n\r\n",
+	              (intmax_t)size);
+
+	if( send_data(h, str.data, str.off, MSG_MORE) == 0 )
 	{
 		if( h->req_command != EHead )
 			send_file(h, fd, 0, size-1);
@@ -1432,16 +1440,15 @@ SendResp_albumArt(struct upnphttp * h, char * object)
 	CloseSocket_upnphttp(h);
 }
 
-void
+static void
 SendResp_caption(struct upnphttp * h, char * object)
 {
 	char header[512];
 	char *path;
-	char date[30];
-	time_t curtime = time(NULL);
 	off_t size;
 	long long id;
-	int fd, ret;
+	int fd;
+	struct string_s str;
 
 	id = strtoll(object, NULL, 10);
 
@@ -1465,17 +1472,12 @@ SendResp_caption(struct upnphttp * h, char * object)
 	size = lseek(fd, 0, SEEK_END);
 	lseek(fd, 0, SEEK_SET);
 
-	strftime(date, 30,"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&curtime));
-	ret = snprintf(header, sizeof(header), "HTTP/1.1 200 OK\r\n"
-	                                       "Content-Type: smi/caption\r\n"
-	                                       "Content-Length: %jd\r\n"
-	                                       "Connection: close\r\n"
-	                                       "Date: %s\r\n"
-	                                       "EXT:\r\n"
-	                                       "Server: " MINIDLNA_SERVER_STRING "\r\n\r\n",
-	                                       (intmax_t)size, date);
+	INIT_STR(str, header);
 
-	if( send_data(h, header, ret, MSG_MORE) == 0 )
+	start_dlna_header(&str, 200, "Interactive", "smi/caption");
+	strcatf(&str, "Content-Length: %jd\r\n\r\n", (intmax_t)size);
+
+	if( send_data(h, str.data, str.off, MSG_MORE) == 0 )
 	{
 		if( h->req_command != EHead )
 			send_file(h, fd, 0, size-1);
@@ -1484,17 +1486,15 @@ SendResp_caption(struct upnphttp * h, char * object)
 	CloseSocket_upnphttp(h);
 }
 
-void
+static void
 SendResp_thumbnail(struct upnphttp * h, char * object)
 {
 	char header[512];
 	char *path;
-	char date[30];
-	time_t curtime = time(NULL);
 	long long id;
-	int ret;
 	ExifData *ed;
 	ExifLoader *l;
+	struct string_s str;
 
 	if( h->reqflags & (FLAG_XFERSTREAMING|FLAG_RANGE) )
 	{
@@ -1534,20 +1534,15 @@ SendResp_thumbnail(struct upnphttp * h, char * object)
 			exif_data_unref(ed);
 		return;
 	}
-	strftime(date, 30,"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&curtime));
-	ret = snprintf(header, sizeof(header), "HTTP/1.1 200 OK\r\n"
-	                                       "Content-Type: image/jpeg\r\n"
-	                                       "Content-Length: %jd\r\n"
-	                                       "Connection: close\r\n"
-	                                       "Date: %s\r\n"
-	                                       "EXT:\r\n"
-	                                       "realTimeInfo.dlna.org: DLNA.ORG_TLAG=*\r\n"
-	                                       "contentFeatures.dlna.org: DLNA.ORG_PN=JPEG_TN;DLNA.ORG_CI=1\r\n"
-	                                       "Server: " MINIDLNA_SERVER_STRING "\r\n"
-	                                       "transferMode.dlna.org: Interactive\r\n\r\n",
-	                                       (intmax_t)ed->size, date);
 
-	if( send_data(h, header, ret, MSG_MORE) == 0 )
+	INIT_STR(str, header);
+
+	start_dlna_header(&str, 200, "Interactive", "image/jpeg");
+	strcatf(&str, "Content-Length: %jd\r\n"
+	              "contentFeatures.dlna.org: DLNA.ORG_PN=JPEG_TN;DLNA.ORG_CI=1\r\n\r\n",
+	              (intmax_t)ed->size);
+
+	if( send_data(h, str.data, str.off, MSG_MORE) == 0 )
 	{
 		if( h->req_command != EHead )
 			send_data(h, (char *)ed->data, ed->size, 0);
@@ -1556,17 +1551,15 @@ SendResp_thumbnail(struct upnphttp * h, char * object)
 	CloseSocket_upnphttp(h);
 }
 
-void
+static void
 SendResp_resizedimg(struct upnphttp * h, char * object)
 {
 	char header[512];
 	char buf[128];
 	struct string_s str;
 	char **result;
-	char date[30];
 	char dlna_pn[22];
 	uint32_t dlna_flags = DLNA_FLAG_DLNA_V1_5|DLNA_FLAG_HTTP_STALLING|DLNA_FLAG_TM_B|DLNA_FLAG_TM_I;
-	time_t curtime = time(NULL);
 	int width=640, height=480, dstw, dsth, size;
 	int srcw, srch;
 	unsigned char * data = NULL;
@@ -1575,12 +1568,12 @@ SendResp_resizedimg(struct upnphttp * h, char * object)
 	char *key, *val;
 	char *saveptr, *item = NULL;
 	int rotate;
-	/* Not implemented yet *
-	char *pixelshape=NULL; */
+	int pixw = 0, pixh = 0;
 	long long id;
 	int rows=0, chunked, ret;
 	image_s *imsrc = NULL, *imdst = NULL;
 	int scale = 1;
+	const char *tmode;
 
 	id = strtoll(object, &saveptr, 10);
 	snprintf(buf, sizeof(buf), "SELECT PATH, RESOLUTION, ROTATION from DETAILS where ID = '%lld'", (long long)id);
@@ -1628,16 +1621,17 @@ SendResp_resizedimg(struct upnphttp * h, char * object)
 			rotate = (rotate + atoi(val)) % 360;
 			sql_exec(db, "UPDATE DETAILS set ROTATION = %d where ID = %lld", rotate, id);
 		}
-		/* Not implemented yet *
 		else if( strcasecmp(key, "pixelshape") == 0 )
 		{
-			pixelshape = val;
-		} */
+			ret = sscanf(val, "%d:%d", &pixw, &pixh);
+			if( ret != 2 )
+				pixw = pixh = 0;
+		}
 	}
 
 #if USE_FORK
 	pid_t newpid = 0;
-	newpid = process_fork();
+	newpid = process_fork(h->req_client);
 	if( newpid > 0 )
 	{
 		CloseSocket_upnphttp(h);
@@ -1652,6 +1646,8 @@ SendResp_resizedimg(struct upnphttp * h, char * object)
 	}
 
 	DPRINTF(E_INFO, L_HTTP, "Serving resized image for ObjectId: %lld [%s]\n", id, file_path);
+	if( rotate )
+		DPRINTF(E_DEBUG, L_HTTP, "Rotating image %d degrees\n", rotate);
 	switch( rotate )
 	{
 		case 90:
@@ -1684,6 +1680,14 @@ SendResp_resizedimg(struct upnphttp * h, char * object)
 		dsth = height;
 		dstw = (((height<<10)/srch) * srcw>>10);
 	}
+	/* Account for pixel shape */
+	if( pixw && pixh )
+	{
+		if( pixh > pixw )
+			dsth = dsth * pixw / pixh;
+		else if( pixw > pixh )
+			dstw = dstw * pixh / pixw;
+	}
 
 	if( dstw <= 160 && dsth <= 160 )
 		strcpy(dlna_pn, "DLNA.ORG_PN=JPEG_TN;");
@@ -1701,26 +1705,17 @@ SendResp_resizedimg(struct upnphttp * h, char * object)
 	else if( srcw>>2 >= dstw && srch>>2 >= dsth )
 		scale = 2;
 
-	str.data = header;
-	str.size = sizeof(header);
-	str.off = 0;
+	INIT_STR(str, header);
 
-	strftime(date, 30,"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&curtime));
-	strcatf(&str, "HTTP/1.1 200 OK\r\n"
-	              "Content-Type: image/jpeg\r\n"
-	              "Connection: close\r\n"
-	              "Date: %s\r\n"
-	              "EXT:\r\n"
-	              "realTimeInfo.dlna.org: DLNA.ORG_TLAG=*\r\n"
-	              "contentFeatures.dlna.org: %sDLNA.ORG_CI=%X;DLNA.ORG_FLAGS=%08X%024X\r\n"
-	              "Server: " MINIDLNA_SERVER_STRING "\r\n",
-	              date, dlna_pn, 1, dlna_flags, 0);
 #if USE_FORK
 	if( (h->reqflags & FLAG_XFERBACKGROUND) && (setpriority(PRIO_PROCESS, 0, 19) == 0) )
-		strcatf(&str, "transferMode.dlna.org: Background\r\n");
+		tmode = "Background";
 	else
 #endif
-		strcatf(&str, "transferMode.dlna.org: Interactive\r\n");
+		tmode = "Interactive";
+	start_dlna_header(&str, 200, tmode, "image/jpeg");
+	strcatf(&str, "contentFeatures.dlna.org: %sDLNA.ORG_CI=1;DLNA.ORG_FLAGS=%08X%024X\r\n",
+	              dlna_pn, dlna_flags, 0);
 
 	if( strcmp(h->HttpVer, "HTTP/1.0") == 0 )
 	{
@@ -1786,7 +1781,7 @@ resized_error:
 #endif
 }
 
-void
+static void
 SendResp_dlnafile(struct upnphttp *h, char *object)
 {
 	char header[1024];
@@ -1794,13 +1789,13 @@ SendResp_dlnafile(struct upnphttp *h, char *object)
 	char buf[128];
 	char **result;
 	int rows, ret;
-	char date[30];
-	time_t curtime = time(NULL);
 	off_t total, offset, size;
 	int64_t id;
 	int sendfh;
 	uint32_t dlna_flags = DLNA_FLAG_DLNA_V1_5|DLNA_FLAG_HTTP_STALLING|DLNA_FLAG_TM_B;
-	uint32_t cflags = client_types[h->req_client].flags;
+	uint32_t cflags = h->req_client ? h->req_client->type->flags : 0;
+	const char *tmode;
+	enum client_types ctype = h->req_client ? h->req_client->type->type : 0;
 	static struct { int64_t id;
 	                enum client_types client;
 	                char path[PATH_MAX];
@@ -1828,7 +1823,7 @@ SendResp_dlnafile(struct upnphttp *h, char *object)
 			return;
 		}
 	}
-	if( id != last_file.id || h->req_client != last_file.client )
+	if( id != last_file.id || ctype != last_file.client )
 	{
 		snprintf(buf, sizeof(buf), "SELECT PATH, MIME, DLNA_PN from DETAILS where ID = '%lld'", (long long)id);
 		ret = sql_get_table(db, buf, &result, &rows, NULL);
@@ -1847,7 +1842,7 @@ SendResp_dlnafile(struct upnphttp *h, char *object)
 		}
 		/* Cache the result */
 		last_file.id = id;
-		last_file.client = h->req_client;
+		last_file.client = ctype;
 		strncpy(last_file.path, result[3], sizeof(last_file.path)-1);
 		if( result[4] )
 		{
@@ -1860,12 +1855,11 @@ SendResp_dlnafile(struct upnphttp *h, char *object)
 				/* Samsung TV's such as the A750 can natively support many
 				   Xvid/DivX AVI's however, the DLNA server needs the 
 				   mime type to say video/mpeg */
-				else if( h->req_client == ESamsungSeriesA &&
-				         strcmp(last_file.mime+6, "x-msvideo") == 0 )
+				else if( ctype == ESamsungSeriesA && strcmp(last_file.mime+6, "x-msvideo") == 0 )
 					strcpy(last_file.mime+6, "mpeg");
 			}
 			/* ... and Sony BDP-S370 won't play MKV unless we pretend it's a DiVX file */
-			else if( h->req_client == ESonyBDP )
+			else if( ctype == ESonyBDP )
 			{
 				if( strcmp(last_file.mime+6, "x-matroska") == 0 ||
 				    strcmp(last_file.mime+6, "mpeg") == 0 )
@@ -1879,7 +1873,7 @@ SendResp_dlnafile(struct upnphttp *h, char *object)
 		sqlite3_free_table(result);
 	}
 #if USE_FORK
-	newpid = process_fork();
+	newpid = process_fork(h->req_client);
 	if( newpid > 0 )
 	{
 		CloseSocket_upnphttp(h);
@@ -1929,14 +1923,20 @@ SendResp_dlnafile(struct upnphttp *h, char *object)
 	size = lseek(sendfh, 0, SEEK_END);
 	lseek(sendfh, 0, SEEK_SET);
 
-	str.data = header;
-	str.size = sizeof(header);
-	str.off = 0;
+	INIT_STR(str, header);
+
+#if USE_FORK
+	if( (h->reqflags & FLAG_XFERBACKGROUND) && (setpriority(PRIO_PROCESS, 0, 19) == 0) )
+		tmode = "Background";
+	else
+#endif
+	if( strncmp(last_file.mime, "image", 5) == 0 )
+		tmode = "Interactive";
+	else
+		tmode = "Streaming";
+
+	start_dlna_header(&str, (h->reqflags & FLAG_RANGE ? 206 : 200), tmode, last_file.mime);
 
-	strcatf(&str, "HTTP/1.1 20%c OK\r\n"
-	              "Content-Type: %s\r\n",
-	              (h->reqflags & FLAG_RANGE ? '6' : '0'),
-	              last_file.mime);
 	if( h->reqflags & FLAG_RANGE )
 	{
 		if( !h->req_RangeEnd || h->req_RangeEnd == size )
@@ -1971,16 +1971,6 @@ SendResp_dlnafile(struct upnphttp *h, char *object)
 		strcatf(&str, "Content-Length: %jd\r\n", (intmax_t)total);
 	}
 
-#if USE_FORK
-	if( (h->reqflags & FLAG_XFERBACKGROUND) && (setpriority(PRIO_PROCESS, 0, 19) == 0) )
-		strcatf(&str, "transferMode.dlna.org: Background\r\n");
-	else
-#endif
-	if( strncmp(last_file.mime, "image", 5) == 0 )
-		strcatf(&str, "transferMode.dlna.org: Interactive\r\n");
-	else
-		strcatf(&str, "transferMode.dlna.org: Streaming\r\n");
-
 	switch( *last_file.mime )
 	{
 		case 'i':
@@ -2000,15 +1990,9 @@ SendResp_dlnafile(struct upnphttp *h, char *object)
 			              lan_addr[h->iface].str, runtime_vars.port, (long long)id);
 	}
 
-	strftime(date, 30,"%a, %d %b %Y %H:%M:%S GMT" , gmtime(&curtime));
 	strcatf(&str, "Accept-Ranges: bytes\r\n"
-	              "Connection: close\r\n"
-	              "Date: %s\r\n"
-	              "EXT:\r\n"
-	              "realTimeInfo.dlna.org: DLNA.ORG_TLAG=*\r\n"
-	              "contentFeatures.dlna.org: %sDLNA.ORG_OP=%02X;DLNA.ORG_CI=%X;DLNA.ORG_FLAGS=%08X%024X\r\n"
-	              "Server: " MINIDLNA_SERVER_STRING "\r\n\r\n",
-	              date, last_file.dlna, 1, 0, dlna_flags, 0);
+	              "contentFeatures.dlna.org: %sDLNA.ORG_OP=%02X;DLNA.ORG_CI=%X;DLNA.ORG_FLAGS=%08X%024X\r\n\r\n",
+	              last_file.dlna, 1, 0, dlna_flags, 0);
 
 	//DEBUG DPRINTF(E_DEBUG, L_HTTP, "RESPONSE: %s\n", str.data);
 	if( send_data(h, str.data, str.off, MSG_MORE) == 0 )
