@@ -65,12 +65,68 @@
 #include "utils.h"
 #include "upnphttp.h"
 #include "upnpsoap.h"
+#include "containers.h"
 #include "upnpreplyparse.h"
 #include "getifaddr.h"
 #include "scanner.h"
 #include "sql.h"
 #include "log.h"
 
+#ifdef __sparc__ /* Sorting takes too long on slow processors with very large containers */
+# define __SORT_LIMIT if( totalMatches < 10000 )
+#else
+# define __SORT_LIMIT
+#endif
+
+/* Standard Errors:
+ *
+ * errorCode errorDescription Description
+ * --------	---------------- -----------
+ * 401 		Invalid Action 	No action by that name at this service.
+ * 402 		Invalid Args 	Could be any of the following: not enough in args,
+ * 							too many in args, no in arg by that name, 
+ * 							one or more in args are of the wrong data type.
+ * 403 		Out of Sync 	Out of synchronization.
+ * 501 		Action Failed 	May be returned in current state of service
+ * 							prevents invoking that action.
+ * 600-699 	TBD 			Common action errors. Defined by UPnP Forum
+ * 							Technical Committee.
+ * 700-799 	TBD 			Action-specific errors for standard actions.
+ * 							Defined by UPnP Forum working committee.
+ * 800-899 	TBD 			Action-specific errors for non-standard actions. 
+ * 							Defined by UPnP vendor.
+*/
+static void
+SoapError(struct upnphttp * h, int errCode, const char * errDesc)
+{
+	static const char resp[] = 
+		"<s:Envelope "
+		"xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
+		"s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
+		"<s:Body>"
+		"<s:Fault>"
+		"<faultcode>s:Client</faultcode>"
+		"<faultstring>UPnPError</faultstring>"
+		"<detail>"
+		"<UPnPError xmlns=\"urn:schemas-upnp-org:control-1-0\">"
+		"<errorCode>%d</errorCode>"
+		"<errorDescription>%s</errorDescription>"
+		"</UPnPError>"
+		"</detail>"
+		"</s:Fault>"
+		"</s:Body>"
+		"</s:Envelope>";
+
+	char body[2048];
+	int bodylen;
+
+	DPRINTF(E_WARN, L_HTTP, "Returning UPnPError %d: %s\n", errCode, errDesc);
+	bodylen = snprintf(body, sizeof(body), resp, errCode, errDesc);
+	BuildResp2_upnphttp(h, 500, "Internal Server Error", body, bodylen);
+	SendResp_upnphttp(h);
+	CloseSocket_upnphttp(h);
+}
+
 static void
 BuildSendAndCloseSoapResp(struct upnphttp * h,
                           const char * body, int bodylen)
@@ -349,7 +405,7 @@ set_filter_flags(char *filter, struct upnphttp *h)
 {
 	char *item, *saveptr = NULL;
 	uint32_t flags = 0;
-	int samsung = client_types[h->req_client].flags & FLAG_SAMSUNG;
+	int samsung = h->req_client && (h->req_client->type->flags & FLAG_SAMSUNG);
 
 	if( !filter || (strlen(filter) <= 1) ) {
 		/* Not the full 32 bits.  Skip vendor-specific stuff by default. */
@@ -508,7 +564,7 @@ set_filter_flags(char *filter, struct upnphttp *h)
 	return flags;
 }
 
-char *
+static char *
 parse_sort_criteria(char *sortCriteria, int *error)
 {
 	char *order = NULL;
@@ -613,7 +669,7 @@ add_resized_res(int srcw, int srch, int reqw, int reqh, char *dlna_pn,
 	int dstw = reqw;
 	int dsth = reqh;
 
-	if( args->flags & FLAG_NO_RESIZE )
+	if( (args->flags & FLAG_NO_RESIZE) && reqw > 160 && reqh > 160 )
 		return;
 
 	strcatf(args->str, "&lt;res ");
@@ -681,11 +737,38 @@ add_res(char *size, char *duration, char *bitrate, char *sampleFrequency,
 	                          runtime_vars.port, detailID, ext);
 }
 
-#define COLUMNS "o.REF_ID, o.DETAIL_ID, o.CLASS," \
+static int
+get_child_count(const char *object, struct magic_container_s *magic)
+{
+	int ret;
+
+	if (magic && magic->child_count)
+		ret = sql_get_int_field(db, "SELECT count(*) from %s", magic->child_count);
+	else if (magic && magic->objectid && *(magic->objectid))
+		ret = sql_get_int_field(db, "SELECT count(*) from OBJECTS where PARENT_ID = '%s';", *(magic->objectid));
+	else
+		ret = sql_get_int_field(db, "SELECT count(*) from OBJECTS where PARENT_ID = '%s';", object);
+
+	return (ret > 0) ? ret : 0;
+}
+
+static int
+object_exists(const char *object)
+{
+	int ret;
+	ret = sql_get_int_field(db, "SELECT count(*) from OBJECTS where OBJECT_ID = '%q'",
+				strcmp(object, "*") == 0 ? "0" : object);
+	return (ret > 0);
+}
+
+#define COLUMNS "o.DETAIL_ID, o.CLASS," \
                 " d.SIZE, d.TITLE, d.DURATION, d.BITRATE, d.SAMPLERATE, d.ARTIST," \
                 " d.ALBUM, d.GENRE, d.COMMENT, d.CHANNELS, d.TRACK, d.DATE, d.RESOLUTION," \
-                " d.THUMBNAIL, d.CREATOR, d.DLNA_PN, d.MIME, d.ALBUM_ART, d.DISC "
-#define SELECT_COLUMNS "SELECT o.OBJECT_ID, o.PARENT_ID, " COLUMNS
+                " d.THUMBNAIL, d.CREATOR, d.DLNA_PN, d.MIME, d.ALBUM_ART, d.ROTATION, d.DISC "
+#define SELECT_COLUMNS "SELECT o.OBJECT_ID, o.PARENT_ID, o.REF_ID, " COLUMNS
+
+#define NON_ZERO(x) (x && atoi(x))
+#define IS_ZERO(x) (!x || !atoi(x))
 
 static int
 callback(void *args, int argc, char **argv, char **azColName)
@@ -694,7 +777,7 @@ callback(void *args, int argc, char **argv, char **azColName)
 	char *id = argv[0], *parent = argv[1], *refID = argv[2], *detailID = argv[3], *class = argv[4], *size = argv[5], *title = argv[6],
 	     *duration = argv[7], *bitrate = argv[8], *sampleFrequency = argv[9], *artist = argv[10], *album = argv[11],
 	     *genre = argv[12], *comment = argv[13], *nrAudioChannels = argv[14], *track = argv[15], *date = argv[16], *resolution = argv[17],
-	     *tn = argv[18], *creator = argv[19], *dlna_pn = argv[20], *mime = argv[21], *album_art = argv[22];
+	     *tn = argv[18], *creator = argv[19], *dlna_pn = argv[20], *mime = argv[21], *album_art = argv[22], *rotate = argv[23];
 	char dlna_buf[128];
 	const char *ext;
 	struct string_s *str = passed_args->str;
@@ -729,9 +812,7 @@ callback(void *args, int argc, char **argv, char **azColName)
 #endif
 	}
 	passed_args->returned++;
-
-	if( runtime_vars.root_container && strcmp(parent, runtime_vars.root_container) == 0 )
-		parent = "0";
+	passed_args->flags &= ~RESPONSE_FLAGS;
 
 	if( strncmp(class, "item", 4) == 0 )
 	{
@@ -877,7 +958,7 @@ callback(void *args, int argc, char **argv, char **azColName)
 		if( strncmp(id, MUSIC_PLIST_ID, strlen(MUSIC_PLIST_ID)) == 0 ) {
 			track = strrchr(id, '$')+1;
 		}
-		if( track && atoi(track) && (passed_args->filter & FILTER_UPNP_ORIGINALTRACKNUMBER) ) {
+		if( NON_ZERO(track) && (passed_args->filter & FILTER_UPNP_ORIGINALTRACKNUMBER) ) {
 			ret = strcatf(str, "&lt;upnp:originalTrackNumber&gt;%s&lt;/upnp:originalTrackNumber&gt;", track);
 		}
 		if( passed_args->filter & FILTER_RES ) {
@@ -895,7 +976,7 @@ callback(void *args, int argc, char **argv, char **azColName)
 					if( srcw > 640 || srch > 480 )
 						add_resized_res(srcw, srch, 640, 480, "JPEG_SM", detailID, passed_args);
 				}
-				if( !(passed_args->flags & FLAG_RESIZE_THUMBS) && tn && atoi(tn) ) {
+				if( !(passed_args->flags & FLAG_RESIZE_THUMBS) && NON_ZERO(tn) && IS_ZERO(rotate) ) {
 					ret = strcatf(str, "&lt;res protocolInfo=\"http-get:*:%s:%s\"&gt;"
 					                   "http://%s:%d/Thumbnails/%s.jpg"
 					                   "&lt;/res&gt;",
@@ -994,7 +1075,7 @@ callback(void *args, int argc, char **argv, char **azColName)
 				}
 			}
 		}
-		if( album_art && atoi(album_art) )
+		if( NON_ZERO(album_art) )
 		{
 			/* Video and audio album art is handled differently */
 			if( *mime == 'v' && (passed_args->filter & FILTER_RES) && !(passed_args->flags & FLAG_MS_PFS) ) {
@@ -1016,7 +1097,7 @@ callback(void *args, int argc, char **argv, char **azColName)
 				ret = strcatf(str, "&lt;upnp:album&gt;%s&lt;/upnp:album&gt;", "[No Keywords]");
 
 			/* EVA2000 doesn't seem to handle embedded thumbnails */
-			if( !(passed_args->flags & FLAG_RESIZE_THUMBS) && tn && atoi(tn) ) {
+			if( !(passed_args->flags & FLAG_RESIZE_THUMBS) && NON_ZERO(tn) && IS_ZERO(rotate) ) {
 				ret = strcatf(str, "&lt;upnp:albumArtURI&gt;"
 				                   "http://%s:%d/Thumbnails/%s.jpg"
 				                   "&lt;/upnp:albumArtURI&gt;",
@@ -1034,13 +1115,10 @@ callback(void *args, int argc, char **argv, char **azColName)
 	{
 		ret = strcatf(str, "&lt;container id=\"%s\" parentID=\"%s\" restricted=\"1\" ", id, parent);
 		if( passed_args->filter & FILTER_SEARCHABLE ) {
-			ret = strcatf(str, "searchable=\"1\" ");
+			ret = strcatf(str, "searchable=\"%d\" ", check_magic_container(id, passed_args->flags) ? 0 : 1);
 		}
 		if( passed_args->filter & FILTER_CHILDCOUNT ) {
-			int children;
-			ret = sql_get_int_field(db, "SELECT count(*) from OBJECTS where PARENT_ID = '%s';", id);
-			children = (ret > 0) ? ret : 0;
-			ret = strcatf(str, "childCount=\"%d\"", children);
+			ret = strcatf(str, "childCount=\"%d\"", get_child_count(id, check_magic_container(id, passed_args->flags)));
 		}
 		/* If the client calls for BrowseMetadata on root, we have to include our "upnp:searchClass"'s, unless they're filtered out */
 		if( passed_args->requested == 1 && strcmp(id, "0") == 0 && (passed_args->filter & FILTER_UPNP_SEARCHCLASS) ) {
@@ -1066,7 +1144,7 @@ callback(void *args, int argc, char **argv, char **azColName)
 		if( artist && (passed_args->filter & FILTER_UPNP_ARTIST) ) {
 			ret = strcatf(str, "&lt;upnp:artist&gt;%s&lt;/upnp:artist&gt;", artist);
 		}
-		if( album_art && atoi(album_art) && (passed_args->filter & FILTER_UPNP_ALBUMARTURI) ) {
+		if( NON_ZERO(album_art) && (passed_args->filter & FILTER_UPNP_ALBUMARTURI) ) {
 			ret = strcatf(str, "&lt;upnp:albumArtURI ");
 			if( passed_args->filter & FILTER_UPNP_ALBUMARTURI_DLNA_PROFILEID ) {
 				ret = strcatf(str, "dlna:profileID=\"JPEG_TN\" xmlns:dlna=\"urn:schemas-dlna-org:metadata-1-0/\"");
@@ -1103,13 +1181,19 @@ BrowseContentDirectory(struct upnphttp * h, const char * action)
 			"<Result>"
 			"&lt;DIDL-Lite"
 			CONTENT_DIRECTORY_SCHEMAS;
+	struct magic_container_s *magic;
 	char *zErrMsg = NULL;
 	char *sql, *ptr;
 	struct Response args;
 	struct string_s str;
-	int totalMatches;
+	int totalMatches = 0;
 	int ret;
-	char *ObjectID, *Filter, *BrowseFlag, *SortCriteria;
+	const char *ObjectID, *BrowseFlag;
+	char *Filter, *SortCriteria;
+	const char *objectid_sql = "o.OBJECT_ID";
+	const char *parentid_sql = "o.PARENT_ID";
+	const char *refid_sql = "o.REF_ID";
+	char where[256] = "";
 	char *orderBy = NULL;
 	struct NameValueParserData data;
 	int RequestedCount = 0;
@@ -1166,24 +1250,9 @@ BrowseContentDirectory(struct upnphttp * h, const char * action)
 
 	args.returned = 0;
 	args.requested = RequestedCount;
-	args.client = client_types[h->req_client].type;
-	args.flags = client_types[h->req_client].flags;
+	args.client = h->req_client ? h->req_client->type->type : 0;
+	args.flags = h->req_client ? h->req_client->type->flags : 0;
 	args.str = &str;
-	if( args.flags & FLAG_MS_PFS )
-	{
-		if( !strchr(ObjectID, '$') && (strcmp(ObjectID, "0") != 0) )
-		{
-			ptr = sql_get_text_field(db, "SELECT OBJECT_ID from OBJECTS"
-			                             " where OBJECT_ID in "
-			                             "('"MUSIC_ID"$%q', '"VIDEO_ID"$%q', '"IMAGE_ID"$%q')",
-			                             ObjectID, ObjectID, ObjectID);
-			if( ptr )
-			{
-				ObjectID = ptr;
-				args.flags |= FLAG_FREE_OBJECT_ID;
-			}
-		}
-	}
 	DPRINTF(E_DEBUG, L_HTTP, "Browsing ContentDirectory:\n"
 	                         " * ObjectID: %s\n"
 	                         " * Count: %d\n"
@@ -1194,47 +1263,65 @@ BrowseContentDirectory(struct upnphttp * h, const char * action)
 				ObjectID, RequestedCount, StartingIndex,
 	                        BrowseFlag, Filter, SortCriteria);
 
-	if( strcmp(ObjectID, "0") == 0 )
-	{
-		args.flags |= FLAG_ROOT_CONTAINER;
-		if( runtime_vars.root_container )
-		{
-			if( (args.flags & FLAG_AUDIO_ONLY) && (strcmp(runtime_vars.root_container, BROWSEDIR_ID) == 0) )
-				ObjectID = MUSIC_DIR_ID;
-			else
-				ObjectID = runtime_vars.root_container;
-		}
-		else
-		{
-			if( args.flags & FLAG_AUDIO_ONLY )
-				ObjectID = MUSIC_ID;
-		}
-	}
-
 	if( strcmp(BrowseFlag+6, "Metadata") == 0 )
 	{
+		const char *id = ObjectID;
 		args.requested = 1;
-		sql = sqlite3_mprintf("SELECT %s, " COLUMNS
-		                      "from OBJECTS o left join DETAILS d on (d.ID = o.DETAIL_ID)"
-	        	              " where OBJECT_ID = '%q';",
-		                      (args.flags & FLAG_ROOT_CONTAINER) ? "0, -1" : "o.OBJECT_ID, o.PARENT_ID",
-		                      ObjectID);
+		magic = in_magic_container(ObjectID, args.flags, &id);
+		if (magic)
+		{
+			if (magic->objectid_sql && strcmp(id, ObjectID) != 0)
+				objectid_sql = magic->objectid_sql;
+			if (magic->parentid_sql && strcmp(id, ObjectID) != 0)
+				parentid_sql = magic->parentid_sql;
+			if (magic->refid_sql)
+				refid_sql = magic->refid_sql;
+		}
+		sql = sqlite3_mprintf("SELECT %s, %s, %s, " COLUMNS
+				      "from OBJECTS o left join DETAILS d on (d.ID = o.DETAIL_ID)"
+				      " where OBJECT_ID = '%q';",
+				      objectid_sql, parentid_sql, refid_sql, id);
 		ret = sqlite3_exec(db, sql, callback, (void *) &args, &zErrMsg);
 		totalMatches = args.returned;
 	}
 	else
 	{
-		ret = sql_get_int_field(db, "SELECT count(*) from OBJECTS where PARENT_ID = '%q'", ObjectID);
-		totalMatches = (ret > 0) ? ret : 0;
-		ret = 0;
-		if( SortCriteria )
+		magic = check_magic_container(ObjectID, args.flags);
+		if (magic)
 		{
-#ifdef __sparc__ /* Sorting takes too long on slow processors with very large containers */
-			if( totalMatches < 10000 )
-#endif
+			if (magic->objectid && *(magic->objectid))
+				ObjectID = *(magic->objectid);
+			if (magic->objectid_sql)
+				objectid_sql = magic->objectid_sql;
+			if (magic->parentid_sql)
+				parentid_sql = magic->parentid_sql;
+			if (magic->refid_sql)
+				refid_sql = magic->refid_sql;
+			if (magic->where)
+				strncpyt(where, magic->where, sizeof(where));
+			if (magic->orderby && !GETFLAG(DLNA_STRICT_MASK))
+				orderBy = strdup(magic->orderby);
+			if (magic->max_count > 0)
+			{
+				int limit = MAX(magic->max_count - StartingIndex, 0);
+				ret = get_child_count(ObjectID, magic);
+				totalMatches = MIN(ret, limit);
+				if (RequestedCount > limit || RequestedCount < 0)
+					RequestedCount = limit;
+			}
+		}
+		if (!where[0])
+			sqlite3_snprintf(sizeof(where), where, "PARENT_ID = '%q'", ObjectID);
+
+		if (!totalMatches)
+			totalMatches = get_child_count(ObjectID, magic);
+		ret = 0;
+		if (SortCriteria && !orderBy)
+		{
+			__SORT_LIMIT
 			orderBy = parse_sort_criteria(SortCriteria, &ret);
 		}
-		else
+		else if (!orderBy)
 		{
 			if( strncmp(ObjectID, MUSIC_PLIST_ID, strlen(MUSIC_PLIST_ID)) == 0 )
 			{
@@ -1245,11 +1332,12 @@ BrowseContentDirectory(struct upnphttp * h, const char * action)
 			}
 			else if( args.flags & FLAG_FORCE_SORT )
 			{
-#ifdef __sparc__
-				if( totalMatches < 10000 )
-#endif
+				__SORT_LIMIT
 				ret = xasprintf(&orderBy, "order by o.CLASS, d.DISC, d.TRACK, d.TITLE");
 			}
+			/* LG TV ordering bug */
+			else if( args.client == ELGDevice )
+				ret = xasprintf(&orderBy, "order by o.CLASS, d.TITLE");
 			else
 				orderBy = parse_sort_criteria(SortCriteria, &ret);
 			if( ret == -1 )
@@ -1266,10 +1354,11 @@ BrowseContentDirectory(struct upnphttp * h, const char * action)
 			goto browse_error;
 		}
 
-		sql = sqlite3_mprintf( SELECT_COLUMNS
+		sql = sqlite3_mprintf("SELECT %s, %s, %s, " COLUMNS
 		                      "from OBJECTS o left join DETAILS d on (d.ID = o.DETAIL_ID)"
-				      " where PARENT_ID = '%q' %s limit %d, %d;",
-				      ObjectID, THISORNUL(orderBy), StartingIndex, RequestedCount);
+				      " where %s %s limit %d, %d;",
+				      objectid_sql, parentid_sql, refid_sql,
+				      where, THISORNUL(orderBy), StartingIndex, RequestedCount);
 		DPRINTF(E_DEBUG, L_HTTP, "Browse SQL: %s\n", sql);
 		ret = sqlite3_exec(db, sql, callback, (void *) &args, &zErrMsg);
 	}
@@ -1284,8 +1373,7 @@ BrowseContentDirectory(struct upnphttp * h, const char * action)
 	/* Does the object even exist? */
 	if( !totalMatches )
 	{
-		ret = sql_get_int_field(db, "SELECT count(*) from OBJECTS where OBJECT_ID = '%q'", ObjectID);
-		if( ret <= 0 )
+		if( !object_exists(ObjectID) )
 		{
 			SoapError(h, 701, "No such object error");
 			goto browse_error;
@@ -1300,13 +1388,11 @@ BrowseContentDirectory(struct upnphttp * h, const char * action)
 	BuildSendAndCloseSoapResp(h, str.data, str.off);
 browse_error:
 	ClearNameValueList(&data);
-	if( args.flags & FLAG_FREE_OBJECT_ID )
-		sqlite3_free(ObjectID);
 	free(orderBy);
 	free(str.data);
 }
 
-inline void
+static inline void
 charcat(struct string_s *str, char c)
 {
 	if (str->size <= str->off)
@@ -1586,13 +1672,15 @@ SearchContentDirectory(struct upnphttp * h, const char * action)
 			"<Result>"
 			"&lt;DIDL-Lite"
 			CONTENT_DIRECTORY_SCHEMAS;
+	struct magic_container_s *magic;
 	char *zErrMsg = NULL;
 	char *sql, *ptr;
 	struct Response args;
 	struct string_s str;
 	int totalMatches;
 	int ret;
-	char *ContainerID, *Filter, *SearchCriteria, *SortCriteria;
+	const char *ContainerID;
+	char *Filter, *SearchCriteria, *SortCriteria;
 	char *orderBy = NULL, *where = NULL, sep[] = "$*";
 	char groupBy[] = "group by DETAIL_ID";
 	struct NameValueParserData data;
@@ -1638,24 +1726,9 @@ SearchContentDirectory(struct upnphttp * h, const char * action)
 
 	args.returned = 0;
 	args.requested = RequestedCount;
-	args.client = client_types[h->req_client].type;
-	args.flags = client_types[h->req_client].flags;
+	args.client = h->req_client ? h->req_client->type->type : 0;
+	args.flags = h->req_client ? h->req_client->type->flags : 0;
 	args.str = &str;
-	if( args.flags & FLAG_MS_PFS )
-	{
-		if( !strchr(ContainerID, '$') && (strcmp(ContainerID, "0") != 0) )
-		{
-			ptr = sql_get_text_field(db, "SELECT OBJECT_ID from OBJECTS"
-			                             " where OBJECT_ID in "
-			                             "('"MUSIC_ID"$%q', '"VIDEO_ID"$%q', '"IMAGE_ID"$%q')",
-			                             ContainerID, ContainerID, ContainerID);
-			if( ptr )
-			{
-				ContainerID = ptr;
-				args.flags |= FLAG_FREE_OBJECT_ID;
-			}
-		}
-	}
 	DPRINTF(E_DEBUG, L_HTTP, "Searching ContentDirectory:\n"
 	                         " * ObjectID: %s\n"
 	                         " * Count: %d\n"
@@ -1666,8 +1739,12 @@ SearchContentDirectory(struct upnphttp * h, const char * action)
 				ContainerID, RequestedCount, StartingIndex,
 	                        SearchCriteria, Filter, SortCriteria);
 
+	magic = check_magic_container(ContainerID, args.flags);
+	if (magic && magic->objectid && *(magic->objectid))
+		ContainerID = *(magic->objectid);
+
 	if( strcmp(ContainerID, "0") == 0 )
-		ContainerID[0] = '*';
+		ContainerID = "*";
 
 	if( strcmp(ContainerID, MUSIC_ALL_ID) == 0 ||
 	    GETFLAG(DLNA_STRICT_MASK) )
@@ -1692,19 +1769,15 @@ SearchContentDirectory(struct upnphttp * h, const char * action)
 	/* Does the object even exist? */
 	if( !totalMatches )
 	{
-		ret = sql_get_int_field(db, "SELECT count(*) from OBJECTS where OBJECT_ID = '%q'",
-		                        !strcmp(ContainerID, "*")?"0":ContainerID);
-		if( ret <= 0 )
+		if( !object_exists(ContainerID) )
 		{
 			SoapError(h, 710, "No such container");
 			goto search_error;
 		}
 	}
-#ifdef __sparc__ /* Sorting takes too long on slow processors with very large containers */
 	ret = 0;
-	if( totalMatches < 10000 )
-#endif
-		orderBy = parse_sort_criteria(SortCriteria, &ret);
+	__SORT_LIMIT
+	orderBy = parse_sort_criteria(SortCriteria, &ret);
 	/* If it's a DLNA client, return an error for bad sort criteria */
 	if( ret < 0 && ((args.flags & FLAG_DLNA) || GETFLAG(DLNA_STRICT_MASK)) )
 	{
@@ -1740,8 +1813,6 @@ SearchContentDirectory(struct upnphttp * h, const char * action)
 	BuildSendAndCloseSoapResp(h, str.data, str.off);
 search_error:
 	ClearNameValueList(&data);
-	if( args.flags & FLAG_FREE_OBJECT_ID )
-		sqlite3_free(ContainerID);
 	free(orderBy);
 	free(where);
 	free(str.data);
@@ -1802,22 +1873,41 @@ SamsungGetFeatureList(struct upnphttp * h, const char * action)
 	static const char resp[] =
 		"<u:X_GetFeatureListResponse xmlns:u=\"urn:schemas-upnp-org:service:ContentDirectory:1\">"
 		"<FeatureList>"
-		"<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n"
 		"&lt;Features xmlns=\"urn:schemas-upnp-org:av:avs\""
 		" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
 		" xsi:schemaLocation=\"urn:schemas-upnp-org:av:avs http://www.upnp.org/schemas/av/avs.xsd\"&gt;"
 		"&lt;Feature name=\"samsung.com_BASICVIEW\" version=\"1\"&gt;"
-		 "&lt;container id=\"" MUSIC_ID "\" type=\"object.item.audioItem\"/&gt;"
-		 "&lt;container id=\"" VIDEO_ID "\" type=\"object.item.videoItem\"/&gt;"
-		 "&lt;container id=\"" IMAGE_ID "\" type=\"object.item.imageItem\"/&gt;"
+		 "&lt;container id=\"%s\" type=\"object.item.audioItem\"/&gt;"
+		 "&lt;container id=\"%s\" type=\"object.item.videoItem\"/&gt;"
+		 "&lt;container id=\"%s\" type=\"object.item.imageItem\"/&gt;"
 		"&lt;/Feature&gt;"
 		"&lt;/Features&gt;"
 		"</FeatureList></u:X_GetFeatureListResponse>";
+	const char *audio = MUSIC_ID;
+	const char *video = VIDEO_ID;
+	const char *image = IMAGE_ID;
+	char body[1024];
+	int len;
 
 	if (runtime_vars.root_container)
-		return SoapError(h, 401, "Invalid Action");
+	{
+		if (strcmp(runtime_vars.root_container, BROWSEDIR_ID) == 0)
+		{
+			audio = MUSIC_DIR_ID;
+			video = VIDEO_DIR_ID;
+			image = IMAGE_DIR_ID;
+		}
+		else
+		{
+			audio = runtime_vars.root_container;
+			video = runtime_vars.root_container;
+			image = runtime_vars.root_container;
+		}
+	}
 
-	BuildSendAndCloseSoapResp(h, resp, sizeof(resp)-1);
+	len = snprintf(body, sizeof(body), resp, audio, video, image);
+
+	BuildSendAndCloseSoapResp(h, body, len);
 }
 
 static void
@@ -1910,52 +2000,3 @@ ExecuteSoapAction(struct upnphttp * h, const char * action, int n)
 	SoapError(h, 401, "Invalid Action");
 }
 
-/* Standard Errors:
- *
- * errorCode errorDescription Description
- * --------	---------------- -----------
- * 401 		Invalid Action 	No action by that name at this service.
- * 402 		Invalid Args 	Could be any of the following: not enough in args,
- * 							too many in args, no in arg by that name, 
- * 							one or more in args are of the wrong data type.
- * 403 		Out of Sync 	Out of synchronization.
- * 501 		Action Failed 	May be returned in current state of service
- * 							prevents invoking that action.
- * 600-699 	TBD 			Common action errors. Defined by UPnP Forum
- * 							Technical Committee.
- * 700-799 	TBD 			Action-specific errors for standard actions.
- * 							Defined by UPnP Forum working committee.
- * 800-899 	TBD 			Action-specific errors for non-standard actions. 
- * 							Defined by UPnP vendor.
-*/
-void
-SoapError(struct upnphttp * h, int errCode, const char * errDesc)
-{
-	static const char resp[] = 
-		"<s:Envelope "
-		"xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
-		"s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
-		"<s:Body>"
-		"<s:Fault>"
-		"<faultcode>s:Client</faultcode>"
-		"<faultstring>UPnPError</faultstring>"
-		"<detail>"
-		"<UPnPError xmlns=\"urn:schemas-upnp-org:control-1-0\">"
-		"<errorCode>%d</errorCode>"
-		"<errorDescription>%s</errorDescription>"
-		"</UPnPError>"
-		"</detail>"
-		"</s:Fault>"
-		"</s:Body>"
-		"</s:Envelope>";
-
-	char body[2048];
-	int bodylen;
-
-	DPRINTF(E_WARN, L_HTTP, "Returning UPnPError %d: %s\n", errCode, errDesc);
-	bodylen = snprintf(body, sizeof(body), resp, errCode, errDesc);
-	BuildResp2_upnphttp(h, 500, "Internal Server Error", body, bodylen);
-	SendResp_upnphttp(h);
-	CloseSocket_upnphttp(h);
-}
-
